//
//  STViewController.m
//  Whereami
//
//  Created by Socratis Tornaritis on 9/4/13.
//  Copyright (c) 2013 Socratis Tornaritis. All rights reserved.
//

#import "STViewController.h"
#import "STMapPoint.h"
#import <CoreLocation/CLGeocoder.h>

@interface STViewController ()

@end


@implementation STViewController


#pragma mark - "UIViewController methods"

- (void)viewDidLoad {
   [super viewDidLoad];
   NSLog(@"viewDidLoad");
   
	// Do any additional setup after loading the view, typically from a nib.
   [[self worldView] setShowsUserLocation:YES];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (id)initWithNibName:(NSString *)nibNameOrNil
               bundle:(NSBundle *)nibBundleOrNil {
   NSLog(@"initWithNibName:bundle:");
   
   if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
      // Create location manager object.
      locationManager = [[CLLocationManager alloc] init];
      
      // Set its delegate to self.
      [locationManager setDelegate:self];
      
      /* And we want it to be as accurate as possilbe,
       * regardless of how much time/power it takes.
       */
      [locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
      
      [[self activityIndicator] startAnimating];
   }
   return self;
}

- (void)dealloc {
   /* Since the CLLocationManager object has a __unsafe_unrelated reference to the
    * view controller, we must reset its delegate property to nil.
    */
   [locationManager setDelegate:nil];
}

- (void)updateCoordinateWithDegrees:(CLLocationDegrees)degrees direction:(NSString *)direction {
   // Temporary references to the UI widgets we must update.
   UILabel* dirLabel;
   UITextField* degTextField;
   UITextField* minTextField;
   UITextField* secTextField;
   
   // Are we converting a lat or long?
   if ([direction compare:@"N" options:NSCaseInsensitiveSearch] == NSOrderedSame ||
       [direction compare:@"S" options:NSCaseInsensitiveSearch] == NSOrderedSame) {
      
      // Converting latitude.
      dirLabel = [self latDirection];
      degTextField = [self latDegrees];
      minTextField = [self latMinutes];
      secTextField = [self latSeconds];
      
   } else {
      // Converting longitude.
      dirLabel = [self longDirection];
      degTextField = [self longDegrees];
      minTextField = [self longMinutes];
      secTextField = [self longSeconds];
   }
   
   /* Make positive so we can break it down easily. */
   degrees = fabs(degrees);         // 89.2345
   
   int deg = (int)degrees;          // 89
   
   degrees -= deg;                  // .2345
   degrees *= 60.0;                 // 14.07
   
   int min = (int)(degrees);        // 14
   
   degrees -= min;                  // .07
   double sec = degrees * 60.0;     // 4.2
   
   [dirLabel setText:direction];
   [degTextField setText:[NSString stringWithFormat:@"%i", deg]];
   [minTextField setText:[NSString stringWithFormat:@"%i", min]];
   [secTextField setText:[NSString stringWithFormat:@"%.1f", sec]];
}

- (void)updateLatitude: (CLLocationDegrees)degreesLatitude {
   if (degreesLatitude > 0) {
      /* North. */
      [self updateCoordinateWithDegrees:degreesLatitude direction:@"N"];
   } else {
      /* South. */
      [self updateCoordinateWithDegrees:degreesLatitude direction:@"S"];
   }
}

- (void)updateLongitude: (CLLocationDegrees)degreesLongitude {
   if (degreesLongitude > 0) {
      /* East. */
      [self updateCoordinateWithDegrees:degreesLongitude direction:@"E"];
   } else {
      /* West. */
      [self updateCoordinateWithDegrees:degreesLongitude direction:@"W"];
   }
}

#pragma mark - "CLLocationManagerDelegate methods"




/* This method is used instead of locationManager:didUpdateToLocation:fromLocation
 * that is now deprecated.
 */

//- (void)locationManager:(CLLocationManager *)manager
//     didUpdateLocations:(NSArray *)locations {
//   
//   NSTimeInterval t = [[lastLocation timestamp] timeIntervalSinceNow];
//   
//   /* If location is less than 3 min old, use it, otherwise get new reading. */
//   if (t < -180) {
//      return;
//   }
//   
//   [self foundLocation:lastLocation];
//}

- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error {
   NSLog(@"Could not find location: %@", error);
}


#pragma mark - "MKMapViewDelegate methods"


- (void)mapView:(MKMapView *)mapView
   didUpdateUserLocation:(MKUserLocation *)userLocation {
   NSLog(@"mapView:didUpdateUserLocation");
   
   /* The default behavior of the mapView is to locate user's location, display the map
    * and show blue dot at the location. The map is not zoomed in however.
    */
   
   /* Update the address field and populate the annotation tag. */
   CLLocation* curLocation = [userLocation location];
   
   CLGeocoder* geocoder = [[CLGeocoder alloc] init];
   
   [geocoder reverseGeocodeLocation: curLocation
                  completionHandler: ^void(NSArray *placemarks, NSError *error) {
                     
                     CLLocationCoordinate2D curCoordinate = [curLocation coordinate];
                     
                     /* Update the lat and long fields. */
                     [self updateLatitude:curCoordinate.latitude];
                     [self updateLongitude:curCoordinate.longitude];
                     
                     /* Grab the geocoder information. */
                     CLPlacemark* placemark = [placemarks objectAtIndex:0];
                     
                     NSString* name = [placemark name];
                     NSString* locality = [placemark locality];
                     NSString* administrativeArea = [placemark administrativeArea];
                     
                     NSString* title = [[NSString alloc] initWithFormat:@"%@", name];
                     NSString* subtitle = [[NSString alloc] initWithFormat:@"%@, %@", locality, administrativeArea];
                     
                     NSString* currAddress = [NSString stringWithFormat:@"%@,%@,%@",name, locality, administrativeArea];
                     [[self currentAddressField] setText:currAddress];
                     
                     // Instantiate a custom annotation object.
                     STMapPoint* mp = [[STMapPoint alloc] initWithCoordinate:curCoordinate
                                                                       title:title
                                                                    subtitle:subtitle];
                     
                     // Add the tag.
                     [[self worldView] addAnnotation:mp];

                     // Set the zoom region to a span of 250m by 250m.
                     MKCoordinateRegion zoomRegion = MKCoordinateRegionMakeWithDistance(curCoordinate, 1250, 1250);
                     
                     // Zoom in.
                     [[self worldView] setRegion:zoomRegion animated:YES];
 
                     /* This has no apparent impact. */
                     [[self activityIndicator] stopAnimating];
                  }];
}

@end
